import "./root";
import "annotations.css";
export declare const bk_annotation = "bk-annotation";
export declare const bk_shading = "bk-shading";
export declare const bk_annotation_child = "bk-annotation-child";
