import "./root";
export declare const bk_canvas = "bk-canvas";
export declare const bk_canvas_map = "bk-canvas-map";
export declare const bk_canvas_overlays = "bk-canvas-overlays";
export declare const bk_canvas_events = "bk-canvas-events";
